@@ -1,11 +1,13 @@
 import asyncio
 import os
 from functools import wraps
+from typing import Dict
 
 from .constants import Phase
 from .flow import CloudFlow, _terminate_flow_simplified
 from .helper import (
     cleanup_dt,
+    get_cph_from_response,
     get_phase_from_response,
     get_str_endpoints_from_response,
     jsonify,
@@ -123,6 +125,11 @@ async def status(args):
                     )
                     _other_rows.append(_add_row_fn('Spec', v))
 
+                elif k == 'CPH' and v:
+                    _other_rows.append(
+                        _add_row_fn("Credits Per Hour", JSON(jsonify(v)))
+                    )
+
                 elif k == 'error' and v:
                     _other_rows.append(_add_row_fn(k, f'[red]{v}[red]'))
 
@@ -139,7 +146,7 @@ async def status(args):
             console.print(_t)
 
 
-async def _list_by_phase(phase: str, name: str):
+async def _list_by_phase(phase: str, name: str, labels: Dict[str, str]):
     from rich import box
     from rich.console import Console
     from rich.table import Table
@@ -147,25 +154,45 @@ async def _list_by_phase(phase: str, name: str):
     from .helper import CustomHighlighter
 
     _t = Table(
-        'ID', 'Status', 'Endpoint(s)', 'Created', box=box.ROUNDED, highlight=True
+        'ID',
+        'Status',
+        'Endpoint(s)',
+        'Credits Per Hour',
+        'Created',
+        box=box.ROUNDED,
+        highlight=True,
     )
 
     console = Console(highlighter=CustomHighlighter())
+
+    # If no phase is passed, show all flows that are not in `Deleted` phase
     if phase is None:
-        phase = ','.join([str(Phase.Serving.value), str(Phase.Failed.value)])
+        phase_to_str = lambda phase: str(phase.value)
+        phase = ','.join(
+            [
+                phase_to_str(Phase.Starting),
+                phase_to_str(Phase.Serving),
+                phase_to_str(Phase.Failed),
+                phase_to_str(Phase.Updating),
+                phase_to_str(Phase.Paused),
+            ]
+        )
+    if labels is not None:
+        labels = labels.replace(',', '&')
     phases = phase.split(',')
-    msg = f'[bold]Fetching [green]{phases[0] if len(phases) == 1 else " and ".join(phases)}[/green] flows'
+    msg = f'[bold]Fetching [green]{phases[0] if len(phases) == 1 else ", ".join(phases)}[/green] flows'
     if name:
         msg += f' with name [green]{name}[/green]'
     msg += ' ...'
     with console.status(msg):
-        _result = await CloudFlow().list_all(phase=phase, name=name)
+        _result = await CloudFlow().list_all(phase=phase, name=name, labels=labels)
         if _result and 'flows' in _result:
             for flow in _result['flows']:
                 _t.add_row(
                     flow['id'],
                     get_phase_from_response(flow),
                     get_str_endpoints_from_response(flow),
+                    get_cph_from_response(flow),
                     cleanup_dt(flow['ctime']),
                 )
         console.print(_t)
@@ -174,7 +201,7 @@ async def _list_by_phase(phase: str, name: str):
 
 @asyncify
 async def list(args):
-    await _list_by_phase(args.phase, args.name)
+    await _list_by_phase(args.phase, args.name, args.labels)
 
 
 @asyncify
@@ -182,6 +209,12 @@ async def remove(args):
     from rich import print
     from rich.prompt import Confirm
 
+    if args.phase is not None:
+        _raw_list = await _list_by_phase(args.phase, '', None)
+        flow_id_list = [flow['id'] for flow in _raw_list['flows']]
+        flows_set_diff = set(flow_id_list).difference(args.flows)
+        args.flows.extend(flows_set_diff)
+
     if args.flows == []:
         print('[cyan]Please pass in flow(s) to remove. Exiting...[/cyan]')
         return
@@ -217,7 +250,9 @@ async def remove(args):
                 return
 
         _raw_list = await _list_by_phase(
-            phase=','.join([str(Phase.Serving.value), str(Phase.Failed.value)]), name=''
+            phase=','.join([str(Phase.Serving.value), str(Phase.Failed.value)]),
+            name='',
+            labels=None,
         )
         print('Above are the flows about to be deleted.\n')
 
@@ -231,10 +266,10 @@ async def remove(args):
 
         flow_id_list = [flow['id'] for flow in _raw_list['flows']]
 
-    await _remove_multi(flow_id_list)
+    await _remove_multi(flow_id_list, args.phase)
 
 
-async def _remove_multi(flow_id_list):
+async def _remove_multi(flow_id_list, phase):
     from rich import print
 
     from .helper import get_pbar
@@ -252,7 +287,7 @@ async def _remove_multi(flow_id_list):
             title=f'Removing {num_flows_to_remove} flows...',
         )
 
-        coros = [_terminate_flow_simplified(flow) for flow in flow_id_list]
+        coros = [_terminate_flow_simplified(flow, phase) for flow in flow_id_list]
         counter = 0
         has_failure_task = False
         for coro in asyncio.as_completed(coros):
@@ -362,3 +397,12 @@ async def scale(args):
     await CloudFlow(flow_id=args.flow).scale(
         executor=args.executor, replicas=args.replicas
     )
+
+
+@asyncify
+async def recreate(args):
+    from rich import print
+
+    print(f'Recreating deleted Flow [green]{args.flow}[/green]')
+
+    await CloudFlow(flow_id=args.flow).recreate()
