@@ -2,26 +2,25 @@ import asyncio
 import json
 import logging
 import os
-import shutil
 import sys
-import tempfile
 import threading
 import warnings
-from contextlib import contextmanager
+from datetime import datetime
 from pathlib import Path
-from typing import Dict, Union
+from typing import Dict, List, Union
 from urllib.parse import urlparse
 from urllib.request import Request, urlopen
 
+import aiohttp
 import pkg_resources
 import yaml
+from dateutil import tz
+from hubble.executor.helper import is_valid_docker_uri, is_valid_sandbox_uri
 from packaging.version import Version
 from rich import print
 from rich.highlighter import ReprHighlighter
 from rich.panel import Panel
 
-from .env_helper import EnvironmentVariables, expand_dict
-
 __windows__ = sys.platform == 'win32'
 
 
@@ -149,45 +148,39 @@ def get_pbar(description, disable=False, total=4):
     return pbar, pb_task
 
 
-@contextmanager
-def zipdir(directory: Path) -> Path:
-    _zip_dest = tempfile.mkdtemp()
-    _zip_name = shutil.make_archive(
-        base_name=directory.name,
-        format='zip',
-        root_dir=str(directory),
-    )
-    shutil.move(_zip_name, _zip_dest)
-    yield Path(os.path.join(_zip_dest, os.path.basename(_zip_name)))
-    shutil.rmtree(_zip_dest)
+def is_valid_serverless_uri(uses):
+    """Check to see if the 'uses' is a valid serverless URI.
 
-
-def valid_uri(uses):
+    Serverless notation is JCloud specific, which we need to handle in addition to
+    what Hubble already provides for URI validation.
+    """
     try:
         return urlparse(uses).scheme in (
-            'docker',
-            'jinahub+docker',
-            'jinahub+sandbox',
             'jinahub+serverless',
-            'jinahub',
+            'jinaai+serverless',
         )
     except Exception:
         return False
 
 
-def normalized(path: Union[str, Path], envs: Dict):
+def normalized(path: Union[str, Path]):
     _normalized = True
+
+    if isinstance(path, str):
+        path = Path(path)
     with open(path) as f:
         _flow_dict = yaml.safe_load(f.read())
 
     if 'executors' in _flow_dict:
-        with EnvironmentVariables(envs):
-            expand_dict(_flow_dict, context=envs)
         for executor in _flow_dict['executors']:
             uses = executor.get('uses', None)
             if uses is None:
                 continue
-            elif valid_uri(uses):
+            elif (
+                is_valid_docker_uri(uses)
+                or is_valid_sandbox_uri(uses)
+                or is_valid_serverless_uri(uses)
+            ):
                 continue
             else:
                 _normalized = False
@@ -204,15 +197,103 @@ def remove_prefix(s: str, prefix: str):
     return s[len(prefix) :] if s.startswith(prefix) else s
 
 
-def prepare_flow_model_for_render(response: Dict) -> None:
-    # 'gateway' and 'endpoints' should be mutually exclusive.
-    if response['gateway']:
-        del response['endpoints']
-    elif response['endpoints']:
-        del response['gateway']
+def jsonify(data: Union[Dict, List]) -> str:
+    return (
+        json.dumps(data, indent=2, sort_keys=True)
+        if isinstance(data, (dict, list))
+        else data
+    )
 
-    # Extra handle for 'dashboards' key; the value is a map
-    # but since we only have monitoring dashboard at the moment,
-    # for simplicity we turn the map to a string for display purpose.
-    if response['dashboards']:
-        response['dashboards'] = response['dashboards']['monitoring']
+
+def yamlify(data: Union[Dict, List]) -> str:
+    return (
+        yaml.dump(data, indent=2, sort_keys=False, default_flow_style=False)
+        if isinstance(data, (dict, list))
+        else data
+    )
+
+
+def get_endpoints_from_response(response: Dict) -> Dict:
+    return response.get('status', {}).get('endpoints', {})
+
+
+def get_str_endpoints_from_response(response: Dict) -> str:
+    return json.dumps(get_endpoints_from_response(response))
+
+
+def get_grafana_from_response(response: Dict) -> str:
+    if response:
+        dashboards = response.get('status', {}).get('dashboards', {})
+        if dashboards is None:
+            return ''
+    return dashboards.get('grafana', '')
+
+
+def get_phase_from_response(response: Dict) -> str:
+    return response.get('status', {}).get('phase', '')
+
+
+def get_cph_from_response(response: Dict) -> str:
+    return str(response.get('CPH', {}).get('total', ''))
+
+
+def cleanup_dt(dt) -> str:
+    try:
+        return (
+            datetime.strptime(dt, '%Y-%m-%dT%H:%M:%S.%f%z')
+            .astimezone(tz=tz.tzlocal())
+            .strftime('%d-%b-%Y %H:%M GMT %z')
+        )
+    except ValueError:
+        try:
+            return (
+                datetime.strptime(dt, '%Y-%m-%dT%H:%M:%S%z')
+                .astimezone(tz=tz.tzlocal())
+                .strftime('%d-%b-%Y %H:%M GMT %z')
+            )
+        except ValueError:
+            return dt
+    except Exception as e:
+        return dt
+
+
+def get_dashboard_from_flowid(flow_id: str) -> str:
+    ns = flow_id.split('-')[-1]
+    return f'https://dashboard.wolf.jina.ai/flow/{ns}'
+
+
+def jcloud_logs_from_response(flow_id: str, response: Dict) -> str:
+    if 'jcloud' not in response:
+        return get_dashboard_from_flowid(flow_id)
+    else:
+        try:
+            return response['jcloud']['url']
+        except KeyError:
+            return get_dashboard_from_flowid(flow_id)
+
+
+def get_dict_list_key_path(collection, keys):
+    col = collection
+    for k in keys:
+        try:
+            col = col[k]
+        except (KeyError, IndexError, TypeError) as e:
+            return None
+    return col
+
+
+def get_condition_from_status(status):
+    try:
+        sts = status["status"]
+        conds = sts["conditions"]
+        for c in conds:
+            if c["type"] == "ReadinessSuccessful":
+                return c
+    except KeyError:
+        return None
+
+
+def get_aiohttp_session() -> aiohttp.ClientSession:
+    return aiohttp.ClientSession(
+        connector=aiohttp.TCPConnector(ssl=False, limit=5, force_close=True),
+    )
